<?php

App::uses('Component', 'Controller');

class ImpersonateComponent extends Component {

	protected $_controller = null;

	public $defaultSettings = [
		// Define the model used by the Auth component.
		'userModel' => 'User',
		// Define any additional parameters for the query used to retrieve the
		// user.
		'params' => [
			'conditions' => [
				'User.user_group_id >' => 2
			]
		],
		// Define the query parameters for checking the current user can
		// impersonate other users.
		'canImpersonateParams' => [
			'conditions' => [
				'User.user_group_id <=' => 2
			]
		],
		// Define the redirect route/url to go to after starting to impersonate
		// a user.
		'impersonateRedirect' => '/',
		// Define the redirect route/url to go to after ending an impersonation
		// session.
		'unimpersonateRedirect' => [
			'admin' => true,
			'plugin' => 'ev_core',
			'controller' => 'users',
			'action' => 'index'
		]
	];

	public function initialize(Controller $controller) {
		parent::initialize($controller);
		$this->_controller = $controller;
		$this->settings = Hash::merge($this->defaultSettings, $this->settings);
	}

	/**
	 * Impersonate another user.
	 * @param  int    $userId ID of the user we want to impersonate
	 * @param  array  $params Additional query parameters for user to impersonate
	 * @return void   Redirect
	 */
	public function impersonate($userId, array $params = []) {
		// Make sure we are not already impersonating a user before
		// impersonating someone.
		$impersonator = $this->_controller->Auth->user();
		if (
			$this->_controller->Session->check('Auth.Impersonator') === false
			&& $this->_canImpersonate($impersonator) === true
		) {
			// Get the user to impersonate.
			$this->_controller->loadModel($this->settings['userModel']);
			$params = Hash::merge($this->settings['params'], $params);
			$defaultParams = [
				'conditions' => [
					$this->_controller->{$this->settings['userModel']}->escapeField() => $userId
				]
			];
			$user = $this->_controller->{$this->settings['userModel']}->find('first', Hash::merge($defaultParams, $params));

			if (!empty($user)) {
				// Store the current auth user as the impersonator.
				$this->_controller->Session->write('Auth.Impersonator', $impersonator);

				$fullUser = $this->_controller->{$this->settings['userModel']}->getUserForLogin($user['User']['id']);

				// Change to recognised auth user to the impersonated user.
				$this->_controller->Auth->login($fullUser);

				// Redirect to ensure we are in the impersonated users session.
				return $this->_controller->redirect($this->settings['impersonateRedirect']);

			}
		}

		$this->_controller->Session->setFlash('Unable to impersonate user!', 'flash_fail');
		return $this->_controller->redirect($this->settings['unimpersonateRedirect']);
	}

	/**
	 * Stop impersonating a user and return to the original session.
	 *
	 * @return void Redirects the user
	 */
	public function unimpersonate() {
		if ($this->_controller->Session->check('Auth.Impersonator') === true) {
			// Restore the impersonator's login as the current user.
			$user = $this->_controller->Session->read('Auth.Impersonator');
			$this->_controller->Auth->login($user);
			$user = $this->_controller->Session->delete('Auth.Impersonator');
			return $this->_controller->redirect($this->settings['unimpersonateRedirect']);
		}

		return $this->_controller->Auth->logout();
	}

	/**
	 * Check if the impersonator is allowed to impersonate other users.
	 * @param  array $impersonator User data from the auth session
	 * @return bool
	 */
	protected function _canImpersonate($impersonator) {
		$primaryKey = $this->_controller->{$this->settings['userModel']}->primaryKey;
		$alias = $this->_controller->{$this->settings['userModel']}->alias;
		$defaultParams = [
			'conditions' => [
				$this->_controller->{$this->settings['userModel']}->escapeField() => $impersonator[$alias][$primaryKey]
			]
		];
		return (bool)$this->_controller->{$this->settings['userModel']}->find(
			'first',
			Hash::merge($defaultParams, $this->settings['canImpersonateParams'])
		);
	}

}
